/*************************************************/
/* This class defines an animation with surfaces */
/*************************************************/

/*
This file is part of Tuxworld.

    Tuxworld is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tuxworld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tuxworld.  If not, see <http://www.gnu.org/licenses/>.

Ce fichier fait partie de Tuxworld.

	Tuxworld est un logiciel libre ; vous pouvez le redistribuer ou le
	modifier suivant les termes de la GNU General Public License telle
	que publiée par la Free Software Foundation ; soit la version 3 de
	la licence, soit (à votre gré) toute version ultérieure.

	Tuxworld est distribué dans l'espoir qu'il sera utile, mais SANS
	AUCUNE GARANTIE ; pas même la garantie implicite de
	COMMERCIABILISABILITÉ ni d'ADÉQUATION à UN OBJECTIF PARTICULIER.
	Consultez la GNU General Public License pour plus de détails.

	Vous devez avoir reçu une copie de la GNU General Public License
	en même temps que Tuxworld ; si ce n'est pas le cas,
	consultez <http://www.gnu.org/licenses>.
*/

#ifndef ANIMATION
#define ANIMATION

#include <string>
#include <vector>
#include <memory>

#include "vector2d.h"
#include "graphic.h"
#include "converter.h"
#include "draw.h"
#include "sequence.h"

enum {
   NO_LOOP,
   LOOP,
   UP_DOWN_LOOP
};

class CAnimation : public CGraphic
{
	
   private :
      // list of indexes images
      const char *seq;
      // nbr_seq : number of indexes in the sequence
      const int nbr_seq;
      // offset : to begin on image[offset]
      const int offset;
      // current position in sequence
      int cursor;
      // list of images
      std::vector< std::shared_ptr<CDraw> > Vimages;
      // false : increment animation, true : decrement
      bool up_down;
      // to slide image alpha up, down and stop
      int slide_up;
      // black alpha color
      Color4f slide_blk;
      
   public :
      // orientation : true return images on Y axis with PI angle
      // nbr_images : number of images
      // file : name of file without id
      // coord : animation locate
      // coef_resol : resolution game between 1 = full resolution
      CAnimation(const bool& orientation_, const int& offset_, const int& nbr_images_, const char *seq_, const int& nbr_seq_, const std::string& file_, const SDL_Rect& coord_, const float& coef_resol_);
      //destructor
      ~CAnimation();

      // loop_type : see enum with LOOP, NO_LOOP and UP_DOWN_LOOP
      // step : for a quick or slow animation
      // deep : put animation on Z axis
      // scale : sclae images

      // animate without showing
      void pass(const int& loop_type, const int& step);
      // show without scale
      void show(const int& loop_type, const bool& orientation, const int& step, const float& deep);
      // show with scale
      void show(const int& loop_type, const bool& orientation, const int& step, const TVector2d<float>& scale, const float& deep);
      // to show with a slide black alpha
      bool slide_black(float speed);
};

inline void CAnimation::pass(const int& loop_type, const int& step) {

   // we go to the next image
   up_down ? cursor -= step : cursor += step;

   if(cursor <= 0) {
         cursor = 0;
         up_down = false;
   }

   if(cursor >= nbr_seq)
      switch(loop_type) {

         case LOOP :
            // restart animation
            cursor = 0;
         break;

         case UP_DOWN_LOOP :
            // go down animation
            cursor = nbr_seq - 1;
            up_down = true;
         break;

         case NO_LOOP :
            // we stop at the end of sequence
            cursor = nbr_seq - 1;
         break;
      }
}

#endif
