/*************************************************/
/* This class is used to create the dynamic boss */
/*************************************************/

/*
This file is part of Tuxworld.

    Tuxworld is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tuxworld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tuxworld.  If not, see <http://www.gnu.org/licenses/>.

Ce fichier fait partie de Tuxworld.

	Tuxworld est un logiciel libre ; vous pouvez le redistribuer ou le
	modifier suivant les termes de la GNU General Public License telle
	que publiée par la Free Software Foundation ; soit la version 3 de
	la licence, soit (à votre gré) toute version ultérieure.

	Tuxworld est distribué dans l'espoir qu'il sera utile, mais SANS
	AUCUNE GARANTIE ; pas même la garantie implicite de
	COMMERCIABILISABILITÉ ni d'ADÉQUATION à UN OBJECTIF PARTICULIER.
	Consultez la GNU General Public License pour plus de détails.

	Vous devez avoir reçu une copie de la GNU General Public License
	en même temps que Tuxworld ; si ce n'est pas le cas,
	consultez <http://www.gnu.org/licenses>.
*/

#ifndef DYNAMIC_BOSS
#define DYNAMIC_BOSS

#define MAX_INJURED_TICKS 32

#include "dynamic_object.h"
#include "dynamic_fire.h"
#include "sequence.h"
#include "macros.h"

enum {
   BOSS_POSE,
   BOSS_TURN,
   BOSS_JUMP,
   BOSS_RUN,
   BOSS_SHUT,
   BOSS_ATTACK,
};

class CDynamic_Boss : public CDynamic_Object
{ 
   public :

      // constructor
      CDynamic_Boss(const SDyn_obj& dyn_obj_, const int& nbr_lifes_, const int& max_ticks_time_to_turn_);

      // boss is blinking and injured ?
      bool blink_injured_read();

      // to active the boss
      void activate();

      // boss is active ?
      bool activated();

      virtual int status() = 0;
      virtual void check(std::vector<std::shared_ptr<CDynamic_Fire> >& VFire_, const TVector2d<float>& p_tux_) = 0;

   protected :

      // to jump
      void jump(const TVector2d<float>& jmp);

      // return true when the boss is hurted
      bool check_(std::vector<std::shared_ptr<CDynamic_Fire> >& VFire_, const TVector2d<float>& p_tux_);

      // to stop when boss is running too slow, return true when he stops
      bool slow_to_stop();

      // to run
      void run(const float& v);

      // change value max_ticks_time_to_turn
      void time_swap_eval(const int& max);

   private :

      // to turn the boss front of tux
      void turn(const TVector2d<float>& p_tux_);

      // boss is alive ?
      void death_eval();

      bool injured, active;
      int time_to_turn, max_ticks_time_to_turn;
      unsigned injured_ticks; 

};

inline void CDynamic_Boss::turn(const TVector2d<float>& p_tux_) {
   // boss turn to right or left to follow tux

   if(p_tux_.x < position_read().x) {
      if(!orientation_read())
         orientation_eval();
   }
   else {
      if(orientation_read())
         orientation_eval();
   }
}

#endif
