/**************************************************/
/* This class is used to create an dynamic object */
/**************************************************/

/*
This file is part of Tuxworld.

    Tuxworld is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tuxworld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tuxworld.  If not, see <http://www.gnu.org/licenses/>.

Ce fichier fait partie de Tuxworld.

	Tuxworld est un logiciel libre ; vous pouvez le redistribuer ou le
	modifier suivant les termes de la GNU General Public License telle
	que publiée par la Free Software Foundation ; soit la version 3 de
	la licence, soit (à votre gré) toute version ultérieure.

	Tuxworld est distribué dans l'espoir qu'il sera utile, mais SANS
	AUCUNE GARANTIE ; pas même la garantie implicite de
	COMMERCIABILISABILITÉ ni d'ADÉQUATION à UN OBJECTIF PARTICULIER.
	Consultez la GNU General Public License pour plus de détails.

	Vous devez avoir reçu une copie de la GNU General Public License
	en même temps que Tuxworld ; si ce n'est pas le cas,
	consultez <http://www.gnu.org/licenses>.
*/

#ifndef DYNAMIC_OBJECT

#define DYNAMIC_OBJECT

#include <cmath>
#include <iostream>
#include <memory>
#include <vector>
#include "vector2d.h"

typedef struct s_dyn_obj {
// position, velocity, acceleration, weight
   TVector2d<float> p, v, a, w;
} SDyn_obj;

// run values
enum {
   RUN_RIGHT_ON = 1,
   RUN_RIGHT_OFF = 2,
   RUN_LEFT_ON = 4,
   RUN_LEFT_OFF = 8,
   MASK_RIGHT = 12,
   MASK_LEFT = 3
};

// jump values
enum {
   NOT_JUMP,
   JUMP,
   STOP_JUMP
};

class CDynamic_Object
{
 
   public :
      // constructor
      CDynamic_Object(const SDyn_obj& dyn_obj_, const int& id_life_, const int& nbr_lifes_);

      // return the velocity
      TVector2d<float> velocity_read();

      // return the position
      TVector2d<float> position_read();

      // add acceleration
      void accel(TVector2d<float> a);

      // stop object
      void stop();

      // return id of life object 
      int id_life_read();

      // read orientation running left or right
      bool orientation_read();

      // object is death ?
      bool death_read();

      // is the fire active ?
      int life_read();

      // run or not with right or left direction
      void life_eval();

      // to death the object without deleting the instance
      void death_eval();

      // kill the object
      void killall();

   private : 

      // to calculate with physic classic methods
      friend class CDynamic;

      // identity of life, we start with 0, and when object resurrect, id_life = 1 ...
      int id_life;

      // orientation to left or right
      bool orientation;

   protected :

      // change the orientation
      void orientation_eval();

      SDyn_obj dyn_obj;
      int nbr_lifes;

      bool isdeath;

};

#endif
