#include "init_gameworld.h"

using namespace std;

void init_game_world(s_game &game) {

   // We init sdl video
   if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
      cerr << "tuxworld.cpp : Can't init SDL: " << SDL_GetError() << endl;
      exit_game(game);
   }

   // enable double buffering for opengl
   SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

   // We create main screen surface
   Uint32 flag_fs;

   if(game.fullscreen)
      flag_fs = SDL_FULLSCREEN;
   else
      flag_fs = 0;

   // We set the video mode
   game.screen = SDL_SetVideoMode(SIZE_TEXTURE_W_MAX * game.coef_resol, SIZE_TEXTURE_H_MAX * game.coef_resol, 32, SDL_OPENGL | flag_fs);

   if (!game.screen) {
      cerr << "tuxworld.cpp : Can't set video mode." << SDL_GetError() << endl;
      exit_game(game);
   }

   // set the opengl state
   glEnable(GL_TEXTURE_2D);
   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glViewport(0, 0, SIZE_TEXTURE_W_MAX, SIZE_TEXTURE_H_MAX);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glOrtho(0.0, SIZE_TEXTURE_W_MAX, 0.0, SIZE_TEXTURE_H_MAX, 0.0, 2.0);
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glClearColor(0.0, 0.0, 0.0, 0.0);

   // We init TTF
   if(TTF_Init() == -1) {
      cerr << "init_gameworld.cpp : Can't init TTF." << SDL_GetError() << endl;
      exit_game(game);
   }

   // We load fonts   
   if(!(game.font = TTF_OpenFont(FONTS))) {
      cerr << "init_gameworld.cpp : Can't load fonts." << SDL_GetError() << endl;
      exit_game(game);
   }

   // We load joystick
   try {
      game.OJoystick = new CJoystick();
   }
   catch(string &caught) {
      cerr << "init_gameworld.cpp : Can't load joystick." << endl;
      exit_game(game);
   }
}
