#include "sounds.h"

using namespace std;

CSounds::CSounds(int nbr_channels) {

   // init the thrad for the audio
   SDL_Init( SDL_INIT_AUDIO);

   // Init SDL mixer
   try {

      if( Mix_OpenAudio( 22050, MIX_DEFAULT_FORMAT, 2, 4096 ) == -1 )
         throw string("sounds.cpp : init audio mode error");
   }
   catch(string caught) {
      cerr << caught << endl;
      throw;
   }

   // allocate mixing channels
   Mix_AllocateChannels(nbr_channels);
}

void CSounds::load_sound(const string& file) {

   // load the wave
   try {
      // we create list with sounds
      // note : vector copy objects and destructors are called with bad pointers
      // note : pointers are free at the end of function
      // conclusion : we make a vector on pointers on pointers

      Mix_Chunk *mc = Mix_LoadWAV(file.c_str());      
      if(!mc)     
         throw string("sounds.cpp : can't load the wave : ") + file;

      snds.push_back(mc);


   }
   catch(string caught) {
      cerr << caught << endl;
      throw;
   }
}

void CSounds::load_music(const string& file) {

   // load the wave
   try {
      if(!(music = Mix_LoadMUS(file.c_str())))
         throw string("sounds.cpp : can't load the music");
   }
   catch(string caught) {
      cerr << caught << endl;
      throw;
   }
}

void CSounds::play_sound(const unsigned id, const int channel) {

      try {
         //play the music
         if( Mix_PlayChannel( channel, snds.at(id), 0 ) == -1 )
            throw string("sounds.cpp : can't play the sound");
      }
      catch(string caught) {
         cerr << caught << endl;
         throw;
      }

}

void CSounds::play_music() {

   //if music is not playing
   if( Mix_PlayingMusic() == 0 )
      try {
         //play the music
         if( Mix_PlayMusic( music, -1 ) == -1 )
            throw string("sounds.cpp : can't play the music");
      }
      catch(string caught) {
         cerr << caught << endl;
         throw;
      }
}

CSounds::~CSounds() {
   if(music)
      Mix_FreeMusic( music );
   for(int i = 0; i < snds.size(); ++i)
      Mix_FreeChunk( snds.at(i) );
   snds.clear();
}
