/************************************/
/* This is a platform game with tux */
/************************************/

/*
This file is part of Tuxworld.

    Tuxworld is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Tuxworld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Tuxworld.  If not, see <http://www.gnu.org/licenses/>.

Ce fichier fait partie de Tuxworld.

	Tuxworld est un logiciel libre ; vous pouvez le redistribuer ou le
	modifier suivant les termes de la GNU General Public License telle
	que publiée par la Free Software Foundation ; soit la version 3 de
	la licence, soit (à votre gré) toute version ultérieure.

	Tuxworld est distribué dans l'espoir qu'il sera utile, mais SANS
	AUCUNE GARANTIE ; pas même la garantie implicite de
	COMMERCIABILISABILITÉ ni d'ADÉQUATION à UN OBJECTIF PARTICULIER.
	Consultez la GNU General Public License pour plus de détails.

	Vous devez avoir reçu une copie de la GNU General Public License
	en même temps que Tuxworld ; si ce n'est pas le cas,
	consultez <http://www.gnu.org/licenses>.
*/

#ifndef TUXWORLD
#define TUXWORLD

#include <iostream>
#include <SDL/SDL_image.h>
#include <SDL/SDL_opengl.h>
#include <vector>

#include "draw.h"
#include "timeleft.h"
#include "sounds.h"
#include "joystick.h"
#include "draw_world.h"
#include "draw_digits.h"
#include "dynamic.h"
#include "dynamic_tux.h"
#include "dynamic_ennemy.h"
#include "dynamic_boss.h"
#include "dynamic_coin.h"
#include "dynamic_fire.h"
#include "animation.h"
#include <SDL/SDL_ttf.h>
#include "fonts.h"
#include "box.h"


typedef struct s_game {
   int t_left, t_right, t_jump, t_fire;
   int b_left, b_right, b_jump, b_fire;
   int nbr_ennemies_seq;
   std::vector<TVector2d<int> > Vcoord_explosion;
   int step_sky, step_ground;
   CJoystick *OJoystick;
   std::vector<TVector2d<unsigned> > VSeq_ennemies;
   std::vector<TVector2d<unsigned> > VSeq_coins;
   std::vector<TVector2d<unsigned> > VSeq_powers;
   int clock, clock_ticks;
   float coef_resol;
   bool fullscreen;
   TTF_Font *font; //fonts
   SDL_Surface *screen;
   SDL_Surface *text_surface_score;
   SDL_Surface *text_surface_clock;
   SDL_Rect rect_text_score, rect_score;
   SDL_Rect rect_text_clock, rect_clock;
   SDL_Rect rect_sky;
   SDL_Rect rect_earth;
   SDL_Rect coord_hearts;
   SDL_Rect coord_power;
   SDL_Color color_text;
   CDraw_World *ODrawW_sky;
   std::vector< std::shared_ptr<CDraw_World> > VDrawW_ground;
   //CDraw *ODraw_screen;
   CDraw *ODraw_text_score;
   CDraw *ODraw_text_clock;
   CDraw_Digits *ODraw_digits_score;
   CDraw_Digits *ODraw_digits_clock;
   std::vector<float> resultants_y_earth; //y resultants vectors for the earth
   std::vector<float> ry_earth; // y position points for the earth
   // tux, fire, coins and game.OEnemy objects
   CDynamic_Tux *OTux;
   CDynamic_Boss *OBoss;
   std::vector< std::shared_ptr<CDynamic_Fire> > VFire;
   std::vector< std::shared_ptr<CDynamic_Fire> > VNutts;
   std::vector< std::shared_ptr<CDynamic_Ennemy> > VEnnemy;
   std::vector< std::shared_ptr<CDynamic_Coin> > VCoin;
   std::vector< std::shared_ptr<CDynamic_Coin> > VPower;
   // We create dynamic objects to calculate new positions of tux, fire, ennemy and coins
   CDynamic *OCalculator_tux;
   std::vector< std::shared_ptr<CDynamic> > VCalc_Fire;
   std::vector< std::shared_ptr<CDynamic> > VCalc_Nutts;
   std::vector< std::shared_ptr<CDynamic> > VCalc_Ennemy;
   std::vector< std::shared_ptr<CDynamic> > VCalc_Coin;
   std::vector< std::shared_ptr<CDynamic> > VCalc_Power;
   CDynamic *OCalculator_boss;
   CDynamic **OCalculator_coin;

   // Animations objects
   CAnimation *anim_screen_wait;
   CAnimation *anim_heart;
   CAnimation *anim_power;
   CAnimation *anim_fire;
   CAnimation *anim_nutts;
   CAnimation *anim_coin;
   std::vector< std::shared_ptr<CAnimation> > VAnim_explosion;
   CAnimation *anim_level;
   CAnimation *anim_tux_right;
   CAnimation *anim_supertux_right;
   CAnimation *anim_baby_tux_right;
   CAnimation *anim_ennemy_right; 
   CAnimation *anim_boss_static, *anim_boss_static_injured;
   CAnimation *anim_boss_jump, *anim_boss_jump_injured;
   CAnimation *anim_boss_run, *anim_boss_run_injured;
   CAnimation *anim_boss_shut, *anim_boss_shut_injured;
   CAnimation *anim_tux_death_right;

   CSounds *sounds;
} s_game;

#endif
